import jarray
import ucar
from ucar.ma2 import Range
from java.net import URL
from java.util import ArrayList

# netcdfhelper -- generic class for helping read NetCDF files
# using the version 2 API.
class netcdfhelper:

  def __init__(self, fname):
    """
    fname is the NetCDF file name
    """
    try:
      self.file = ucar.nc2.NetcdfFile(fname)
    except:
      #print "local file: ",fname," not found"
      #print "trying to open as a remote DODS file..."
      try:
        self.file = ucar.nc2.dods.DODSNetcdfFile(fname)
      except:
        #print "dods file: ",fname," not found"
        #print "tyring to open as URL: ",fname
        url = URL(fname)
        self.file = ucar.nc2.NetcdfFile(url)
    

  def close(self):
    self.file.close()

  def getDimensions(self):
    """
    Return a dictionary of the dimenions for each variable
    """
    self.di = self.file.getDimensions().iterator()
    self.dim = {}
    while (self.di.hasNext()):
      self.dit = self.di.next()
      self.dim[self.dit.getName()] = self.dit.getLength()
    return self.dim

  def getAttributes(self):
    """
    Return a list of the attributes
    """
    self.ai = self.file.getGlobalAttributes().iterator()
    self.att = []
    while (self.ai.hasNext()):
      self.att.append(self.ai.next())
    gai = self.file.getRootGroup().getAttributes().iterator()
    while (gai.hasNext()):
      self.att.append(gai.next())
    return self.att

  def getAttribute(self,name):
    """
    Get a single, named attribute
    """
    attr = self.file.findGlobalAttribute(name)
    if attr == None:
      attr = self.file.getRootGroup().findAttribute(name)
    return attr

  def getVariable(self, name):
    """
    Get a single, named variable
    """
    if name != None:
      return self.file.findVariable(name)
    else:
      return None

  def getVariables(self):
    """
    Return a list of all the variables
    """
    self.vi = self.file.getVariables().iterator()
    self.var = []
    while (self.vi.hasNext()):
      self.var.append(self.vi.next())
    return self.var

  def getFloat(self, variable, start):
    """
    Get the value of the variable at the index (may be more than
    """
    size = []
    for i in xrange(len(start)):
      size.append(1)
    array = variable.read(start, size)
    ja = array.copyTo1DJavaArray()
    return float(ja[0])


  def getValues(self, variable, start=None, size=None, stride=None):
    if (stride != None):
      range_list = ArrayList()
      for i in xrange(len(start)):
        rng = Range(start[i], start[i]+(size[i]-1)*stride[i], stride[i])
        range_list.add(i, rng)
      ncdf_array = variable.read(range_list)
      array = ncdf_array.copyTo1DJavaArray()
      return array
    if (start == None and size == None):
      ncdf_array = variable.read()
      array = ncdf_array.copyTo1DJavaArray()
      return array
    else:
      range_list = ArrayList()
      for i in xrange(len(start)):
        rng = Range(start[i], start[i]+(size[i]-1), 1)
        range_list.add(i, rng)
      ncdf_array = variable.read(range_list)
      array = ncdf_array.copyTo1DJavaArray()
      return array




if __name__=='__main__':
    from sys import argv
    fname = (argv[1:])[0]
    print fname
    ncdf = netcdfhelper(fname)
    print ncdf.getDimensions()
    print ncdf.getAttributes()
    var = ncdf.getVariable('EV_1KM_Emissive')
    print var
    vals = ncdf.getValues(var, [0,0,0], [1,500,135], [1,10,10])
    print len(vals)
    val  = ncdf.getFloat(var, [1,100,100])
    print val
    vals = ncdf.getValues(var, [0,100,100], [16,1,1], [1,1,1])
    print len(vals)

    """
    rng_list = ArrayList()
    rng_list.add(0, Range(0, 16, 1))
    rng_list.add(1, Range(100, 100, 1))
    rng_list.add(2, Range(0, 1354, 1))
    sec_var = var.section(rng_list)
    vals = sec_var.read()
    """
